// connection status
var connected = false;

// verbose logging flag
var verbose = false;

// host version for backward compatibility
var hostVersion;

// send a message to the page
function sendPageMessage(message) {
  if (verbose)
    console.log('Sending message [' + JSON.stringify(message) + '] to the page');
  message[ P_SOURCE ] = SRC_EXTN;
  window.postMessage(message, "*");
}

function isOutdatedVersion(version, requiredVersion) {
    return (compareVersionRecursive(version.split("."), requiredVersion.split(".")) < 0);
}

function compareVersionRecursive(a, b) {
    if (a.length == 0) {
        a = [0];
    }
    if (b.length == 0) {
        b = [0];
    }
    if (a[0] != b[0] || (a.length == 1 && b.length == 1)) {
        return a[0] - b[0];
    }

    return compareVersionRecursive(a.slice(1), b.slice(1));
}

// send a message to the extension
function sendExtensionMessage(message) {
  if (verbose)
    console.log('Sending message [' + JSON.stringify(message) + '] to the extension');
  chrome.runtime.sendMessage(message, function(response) {});
}

// load constants
console.log('Loading PEXA Signing constants');

var constants = document.createElement('script');
constants.setAttribute("type","text/javascript");
constants.setAttribute("src", chrome.extension.getURL("scripts/constants.js"));
document.body.appendChild(constants);

// create listeners
console.log('Adding PEXA Signing event listeners');

// post messages from extension to page
chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
  if (request.version !== null && request.version  !== undefined ) {
  	    hostVersion = request.version;
  }

  sendPageMessage(request);
});

// process messages from the page
window.addEventListener("message", function(message) {
  if (message.source !== window) {
    // ignore anything not from our extension
    return;
  }

  if (message.data[ P_STATUS ]) {
    var hostStatus = message.data[ P_STATUS ].toLowerCase();

    console.log('Native host is [' + hostStatus + ']');
    if (hostStatus == S_CONNECTED) {
      connected = true;
    } else if (hostStatus == S_DISCONNECTED) {
      connected = false;
    }
  }

	if (message.data[ P_SOURCE ] && (message.data[ P_SOURCE ] == SRC_PAGE)) {
    // only process messages from the signing page

    if (message.data.verbose !== null)
      verbose = message.data.verbose === "ON" ? true : false;

    if (verbose)
      console.log('Processing message [' + JSON.stringify(message.data) + '] from the page');

    if (message.data[ P_ACTION ]) {
      var action = message.data[ P_ACTION ].toLowerCase();

      if (ACTIONS.indexOf(action) >= 0) {
        if (action == A_VERSION) {
          var manifest = chrome.runtime.getManifest();

          if (verbose)
            console.log('Extension [' + manifest.name + '] is version [' + manifest.version + ']');

          var response = new Object();
          response[ P_ACTION ] = A_VERSION;
          response[ P_RESULT ] = S_OK;
          response[ P_NAME ] = manifest.name;
          response[ P_VERSION ] = manifest.version;
          sendPageMessage(response);
        }
      } else {
        // unsupported action
        console.log('Unsupported [' + P_ACTION + '] [' + action + ']');
        var response = new Object();
        response[ P_ACTION ] = action;
        response[ P_RESULT ] = S_UNSUPPORTED;
        response[ P_REASON ] = 'Unsupported [' + P_ACTION + '] [' + action + ']';
        sendPageMessage(response);
      }
    } else {
      // pass message to native host
      sendExtensionMessage(message.data);
    }
  }
}, false);

window.addEventListener("beforeunload", function(event) {
  if (connected) {
    // host version 1.0.9 and greater rely on timeout to self terminate.
    if (isOutdatedVersion(hostVersion, "1.0.9")) {
      // close the native application
      var disconnect = new Object();
      disconnect[ P_ACTION ] = A_DISCONNECT;
      sendExtensionMessage(disconnect);
      connected = false;
    }
  }
}, false);
